from shapely.ops import cascaded_union

##########################################################################
#  
#  Generation of boundary representation from arbitrary geophysical
#  fields and initialisation for anisotropic, unstructured meshing.
#  
#  Copyright (C) 2011-2013 Dr Adam S. Candy, adam.candy@imperial.ac.uk
#  
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#  
##########################################################################

##########################################################################
#  
#  Generation of boundary representation from arbitrary geophysical
#  fields and initialisation for anisotropic, unstructured meshing.
#  
#  Copyright (C) 2011-2013 Dr Adam S. Candy, adam.candy@imperial.ac.uk
#  
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#  
##########################################################################

import shapefile
from shapely.geometry import *
import sys
import matplotlib.pyplot as pyplot

def getBoundaryPointsList(bounds):
	shapes = bounds.shapes()
	pointsList = []
	for i in range(len(shapes)):
		pointsList.append(shapes[i].points)
	polygons = []
	for j in range(len(pointsList)):
		polygons.append(Polygon([pointsList[j][i] for i in range(len(pointsList[j]))]))
	return (polygons,pointsList)

"""
Work out which objects from the .shp file are overlapping and return the exterior coords of this new shape.
"""
def overlap (bounds, plot = False):
# Read shapefile and work out overlap. Optional plot.
	shapes = bounds.shapes()
	pointsList = []
	for i in range(len(shapes)):
		# Check datapoint is valid.
		pointsList.append(shapes[i].points)

	# Turn the points into polygons.
	polygons = []
	for j in range(len(pointsList)):
		polygons.append(Polygon([pointsList[j][i] for i in range(len(pointsList[j]))]))

	# Add overlapping shapes into a list so we know which to join together.
	overlapping = []
	for n in range(len(polygons) - 1):
		if polygons[n].intersects(polygons[n+1]) == True: 
			# Two if statements to make sure the same polygon isn't being entered more than once.
			if polygons[n] not in overlapping: overlapping.append(polygons[n]) 
			if polygons[n + 1] not in overlapping: overlapping.append(polygons[n + 1]) 

	# Create a new shape from the overlapping shapes.
	join = cascaded_union(overlapping)
	poly = [join]

	# Take the coords. of the perimeter of this new shape.
	coords = []
	for i in range(len(join.exterior.coords)):
		coords.append(list(join.exterior.coords[i]))


	# Plot results if True. Store x-y coords of the perimeter in two lists to plot.
	if plot == True:
		x = []; y = []
		for i in range(len(coords)):
			x.append(coords[i][0]); y.append(coords[i][1])

		# Plot results.
		pyplot.plot(x, y)
		pyplot.xlim(-4, 4)
		pyplot.ylim(-4, 4)
		pyplot.show()
	return poly, coords



"""
Output the final results as a .geo file.
"""
def write_geo (coords, filename):
# Write new shape to .geo.
	print coords

	target = open("%s.geo" % filename, "w") # Creates .geo file to write to.
	for i in range(len(coords)):
		# Write point.
		target.write('Point(%d) = {%.3f, %.3f, 0, %.3f};\n' %(i + 1, coords[i][0], coords[i][1], 1.0))
		# Write the lines connecting the sequential points.
		if (i + 1 > 1): target.write('Line(%d) = {%d, %d};\n' % (i, i, i + 1))
	# Connect first and last points.
	target.write('Line(%d) = {%d, %d};\n' % (i + 1, 1, i + 1))
	target.close()
	return False


"""
Is point in points list
"""
def checkIfPointIsInPointList(points, pt):
	for p in points:
		if p[0]==pt.x and p[1]== pt.y:
			return True
	return False

def checkIfPointIsInPointListOverlap(points, pt):
	if points[0] == pt.x and points[1] == pt.y:
			return True
	return False



"""
	This function checks if the given point is on the boundary.
	@param point : specifies the point which has to be checked
	@param bouds : specifies all the bpundary objects
	@return			 : retucheckPointOnBoundary(p, boundaryPointList)rns true iff the point in on the boundary lines
"""
def checkPointOnBoundary(point, boundaryPoints):
	for points in boundaryPoints:
#		if checkIfPointIsInPointListOverlap(points, p):
		if checkIfPointIsInPointList(points, p):
			return True
	#for s in bounds.shapes():
		for i in range(len(points)-1):
			line = LineString([(points[i][0],points[i][1]),(points[i+1][0], points[i+1][1])])
			if (line.contains(point)):
				return True
	return False




"""
This method returns a list of all the boundary points 
on the boundaries given. This method also returns polygons
consited of the given shapes
@param bounds : specfies the boundary shapes
@return				: returns a tuple containing the polygons shapes
								of the boundary and the exterior points list for
								them
"""



def check_point_within_boundary(p, boundaries):
	for poly in boundaries :
		if p.within(poly):
			return True
	return False


assert len(sys.argv)==4, "Incorrect Number of Arguments passed"

"""
Sets the read and the write file stream according to
the command line arguments given.

The first argument specifies which shape file the user
wants to specify the boundaries of

The second arguments specifies the boundary polygon

The third argument specifies the file path to which the
new shape has top be written

"""

readPath = sys.argv[1]
boundaryPath = sys.argv[2]
writePath = sys.argv[3]

#input stream for the given shape
sf = shapefile.Reader(readPath)

#shapes contained in the given file
shapes = sf.shapes();

#boundary = bounds.shapes[0]
boundary = shapefile.Reader(boundaryPath)

# Create list of the points defined in the .shp file.
pointslist = []
for i in range(len(shapes)):
	# Check datapoint is valid.
	pointslist.append(shapes[i].points)

print(len(boundary.shapes()))
if (len(boundary.shapes()) > 1): boundaryPolygons, boundaryPointList1 = overlap(boundary); boundaryPointList = [boundaryPointList1]
else: boundaryPolygons, boundaryPointList = getBoundaryPointsList(boundary)

i = -1
c = 1
for shape in shapes:
	i += 1
	x =[]; y=[]
	for point in pointslist[i]:
		p = Point(point[0],point[1])
		if check_point_within_boundary(p, boundaryPolygons) or checkPointOnBoundary(p, boundaryPointList):
			x.append(p.x)
			y.append(p.y)
			print(p)
		c +=1
		pyplot.plot(x,y)

pyplot.show()
